@@ -23,6 +23,7 @@
 #include "gchempaint-config.h"
 #include "gchempaint-priv.h"
 #include "gofficeapp.h"
+#include "lib/theme.h"
 
 #include <gsf/gsf-impl-utils.h>
 #include <goffice/app/module-plugin-defs.h>
@@ -96,6 +97,7 @@ go_gchempaint_component_set_data (GOComponent *component)
 {
 	GOGChemPaintComponent *gogcp = GO_GCHEMPAINT_COMPONENT (component);
 	gogcp->document = app->ImportDocument (component->mime_type, component->data, component->length);
+	gcpTheme *pTheme = gogcp->document->GetTheme ();
 	GtkWidget *w = gogcp->document->GetWidget ();
 	gnome_canvas_update_now (GNOME_CANVAS (w));
 	ArtDRect rect;
@@ -103,9 +105,9 @@ go_gchempaint_component_set_data (GOComponent *component)
 	pData->GetObjectBounds (gogcp->document, &rect);
 	double y = gogcp->document->GetYAlign ();
 	y += gogcp->document->GetView ()->GetBaseLineOffset ();
-	y *= pData->ZoomFactor;
+	y *= pTheme->GetZoomFactor ();
 	if (rect.x0 || rect.y0)
-		gogcp->document->Move (- rect.x0 / pData->ZoomFactor, - rect.y0 / pData->ZoomFactor);
+		gogcp->document->Move (- rect.x0 / pTheme->GetZoomFactor (), - rect.y0 / pTheme->GetZoomFactor ());
 	gogcp->document->GetView ()->Update (gogcp->document);
 	if (y < rect.y0)
 		y = rect.y1;
@@ -176,9 +178,9 @@ go_gchempaint_component_edit (GOComponent *component)
 {
 	GOGChemPaintComponent *gogcp = GO_GCHEMPAINT_COMPONENT (component);
 	if (!gogcp->document) {
-		component->ascent = 0.;
+		component->ascent = 1.;
 		component->descent = 0.;
-		component->width = 0.;
+		component->width = 1.;
 	}
 	if (gogcp->window) {
 		gogcp->window->Show ();
@@ -202,6 +204,9 @@ go_gchempaint_component_init (GOComponent *component)
 	component->resizable = false;
 	component->editable = true;
 	component->window = NULL;
+	component->ascent = 1.;
+	component->descent = 0.;
+	component->width = 1.;
 }
 
 static void
