@@ -33,6 +33,7 @@
 #include "reaction.h"
 #include "mesomery.h"
 #include "text.h"
+#include "theme.h"
 #include "window.h"
 #include <locale.h>
 #include <unistd.h>
@@ -49,7 +50,7 @@ using namespace std;
 SignalId OnChangedSignal;
 SignalId OnDeleteSignal;
 
-gcpDocument::gcpDocument(gcpApplication *App, bool StandAlone, gcpWindow *window): Document()
+gcpDocument::gcpDocument(gcpApplication *App, bool StandAlone, gcpWindow *window): Document (App)
 {
 	m_pApp = App;
 	m_pView = NULL;
@@ -58,6 +59,7 @@ gcpDocument::gcpDocument(gcpApplication *App, bool StandAlone, gcpWindow *window
 	m_label = NULL;
 	m_title = NULL;
 	m_bWriteable = true;
+	SetTheme (ThemeManager.GetTheme ("Default"));
 	m_pView = new gcpView(this, !StandAlone);
 	m_bDirty = m_bIsLoading = m_bUndoRedo = false;
 	m_FileType = gcp_ft_Normal;
@@ -180,7 +182,7 @@ bool gcpDocument::ImportOB(OBMol& Mol)
 	OBBond *bond;
 	vector<OBEdgeBase*>::iterator j;
 	bond = Mol.BeginBond(j);
-	double l = (bond)? floor(DefaultBondLength / bond->GetLength()): 1.;
+	double l = (bond)? floor(m_BondLength / bond->GetLength()): 1.;
 	if (l == 0.) l = 1.;
 	m_title = g_strdup(Mol.GetTitle());
 	OBAtom *atom;
@@ -419,13 +421,13 @@ void gcpDocument::AddBond(gcpBond* pBond)
 		while (GetDescendant(id) != NULL);
 		pBond->SetId(id);
 	}
+	AddChild (pBond);
 	gcpAtom *pAtom0 = (gcpAtom*)pBond->GetAtom(0), *pAtom1 = (gcpAtom*)pBond->GetAtom(1);
 	m_pView->Update(pAtom0);
 	m_pView->Update(pAtom1);
 	m_pView->AddObject(pBond);
 	if (m_bIsLoading)
 		return;
-//	AddObject(pBond);
 	//search molecules
 	gcpMolecule * pMol0 = (gcpMolecule*)pAtom0->GetMolecule(),  *pMol1 = (gcpMolecule*)pAtom1->GetMolecule();
 	if (pMol0 && pMol1)
@@ -1128,3 +1130,18 @@ double gcpDocument::GetYAlign ()
 		return (rect.y1 - rect.y0) / 2.;
 	}
 }
+
+void gcpDocument::SetTheme (gcpTheme *theme)
+{
+	m_Theme = theme;
+	m_BondLength = theme->GetBondLength ();
+	m_BondAngle = theme->GetBondAngle ();
+	m_ArrowLength = theme->GetArrowLength ();
+	m_FontFamily = theme->GetFontFamily ();
+	m_TextFontFamily = theme->GetTextFontFamily ();
+	m_FontStyle = theme->GetFontStyle ();
+	m_FontWeight = theme->GetFontWeight ();
+	m_FontVariant = theme->GetFontVariant ();
+	m_FontStretch = theme->GetFontStretch ();
+	m_FontSize = theme->GetFontSize ();
+}
