@@ -81,6 +81,7 @@ struct _GnomeCanvasRichTextExtPrivate {
 
 	int clicks;
 	guint click_timeout;
+	PangoContext *ltr_context, *rtl_context;
 };
 
 enum {
@@ -207,6 +208,14 @@ gnome_canvas_rich_text_ext_finalize(GObject *object)
 
 	text = GNOME_CANVAS_RICH_TEXT_EXT(object);
 
+	if (text->_priv->ltr_context) {
+		g_object_unref (G_OBJECT (text->_priv->ltr_context));
+		text->_priv->ltr_context = NULL;
+	}
+	if (text->_priv->rtl_context) {
+	g_object_unref (G_OBJECT (text->_priv->rtl_context));
+		text->_priv->rtl_context = NULL;
+	}
 	g_free (text->_priv);
 	text->_priv = NULL;
 
@@ -1292,7 +1301,7 @@ gnome_canvas_rich_text_ext_button_press_event(GnomeCanvasItem *item,
 	event_type = event->type;
 	if (event_type == GDK_BUTTON_PRESS) {
 		text->_priv->clicks++;
-		text->_priv->click_timeout = gtk_timeout_add(400, _click, text);
+		text->_priv->click_timeout = g_timeout_add(400, _click, text);
 
 		if (text->_priv->clicks > 3)
 			text->_priv->clicks = text->_priv->clicks % 3;
@@ -1637,10 +1646,10 @@ blink_cb(gpointer data)
 
 	visible = gtk_text_layout_get_cursor_visible(text->_priv->layout);
 	if (visible)
-		text->_priv->blink_timeout = gtk_timeout_add(
+		text->_priv->blink_timeout = g_timeout_add(
 			CURSOR_OFF_TIME, blink_cb, text);
 	else
-		text->_priv->blink_timeout = gtk_timeout_add(
+		text->_priv->blink_timeout = g_timeout_add(
 			CURSOR_ON_TIME, blink_cb, text);
 
 	gtk_text_layout_set_cursor_visible(text->_priv->layout, !visible);
@@ -1660,25 +1669,25 @@ gnome_canvas_rich_text_ext_start_cursor_blink(GnomeCanvasRichTextExt *text,
 		return;
 
 	if (text->_priv->preblink_timeout != 0) {
-		gtk_timeout_remove(text->_priv->preblink_timeout);
+		g_source_remove(text->_priv->preblink_timeout);
 		text->_priv->preblink_timeout = 0;
 	}
 
 	if (with_delay) {
 		if (text->_priv->blink_timeout != 0) {
-			gtk_timeout_remove(text->_priv->blink_timeout);
+			g_source_remove(text->_priv->blink_timeout);
 			text->_priv->blink_timeout = 0;
 		}
 
 		gtk_text_layout_set_cursor_visible(text->_priv->layout, TRUE);
 
-		text->_priv->preblink_timeout = gtk_timeout_add(
+		text->_priv->preblink_timeout = g_timeout_add(
 			PREBLINK_TIME, preblink_cb, text);
 	}
 	else {
 		if (text->_priv->blink_timeout == 0) {
 			gtk_text_layout_set_cursor_visible(text->_priv->layout, TRUE);
-			text->_priv->blink_timeout = gtk_timeout_add(
+			text->_priv->blink_timeout = g_timeout_add(
 				CURSOR_ON_TIME, blink_cb, text);
 		}
 	}
@@ -1688,7 +1697,7 @@ static void
 gnome_canvas_rich_text_ext_stop_cursor_blink(GnomeCanvasRichTextExt *text)
 {
 	if (text->_priv->blink_timeout) {
-		gtk_timeout_remove(text->_priv->blink_timeout);
+		g_source_remove(text->_priv->blink_timeout);
 		text->_priv->blink_timeout = 0;
 	}
 } /* gnome_canvas_rich_text_ext_stop_cursor_blink */
@@ -1721,7 +1730,7 @@ invalidated_handler(GtkTextLayout *layout, gpointer data)
 
 	/* We are called from the update cycle; gotta put this in an idle
 	   loop. */
-	gtk_idle_add(request_update, text);
+	g_idle_add(request_update, text);
 } /* invalidated_handler */
 
 static void
@@ -1771,7 +1780,7 @@ changed_handler(GtkTextLayout *layout, gint start_y,
 
 	/* We are called from the update cycle; gotta put this in an idle
 	   loop. */
-	gtk_idle_add(request_update, text);
+	g_idle_add(request_update, text);
 } /* changed_handler */
 
 void
@@ -1906,7 +1915,6 @@ gnome_canvas_rich_text_ext_ensure_layout(GnomeCanvasRichTextExt *text)
 	if (!text->_priv->layout) {
 		GtkWidget *canvas;
 		GtkTextAttributes *style;
-		PangoContext *ltr_context, *rtl_context;
 
 		text->_priv->layout = gtk_text_layout_new();
 
@@ -1927,17 +1935,14 @@ gnome_canvas_rich_text_ext_ensure_layout(GnomeCanvasRichTextExt *text)
 
 		canvas = GTK_WIDGET(GNOME_CANVAS_ITEM(text)->canvas);
 
-		ltr_context = gtk_widget_create_pango_context(canvas);
-		pango_context_set_base_dir(ltr_context, PANGO_DIRECTION_LTR);
-		rtl_context = gtk_widget_create_pango_context(canvas);
-		pango_context_set_base_dir(rtl_context, PANGO_DIRECTION_RTL);
+		text->_priv->ltr_context = gtk_widget_create_pango_context(canvas);
+		pango_context_set_base_dir(text->_priv->ltr_context, PANGO_DIRECTION_LTR);
+		text->_priv->rtl_context = gtk_widget_create_pango_context(canvas);
+		pango_context_set_base_dir(text->_priv->rtl_context, PANGO_DIRECTION_RTL);
 		gnome_canvas_rich_text_set_cursor_direction (text);
 
 		gtk_text_layout_set_contexts(
-			text->_priv->layout, ltr_context, rtl_context);
-
-		g_object_unref(G_OBJECT(ltr_context));
-		g_object_unref(G_OBJECT(rtl_context));
+			text->_priv->layout, text->_priv->ltr_context, text->_priv->rtl_context);
 
 		style = gtk_text_attributes_new();
 
@@ -2074,7 +2079,11 @@ static void gnome_canvas_rich_text_ext_get_bounds(GnomeCanvasItem *item, double
 	   double *px2, double *py2)
 {
 	GnomeCanvasRichTextExt *text = GNOME_CANVAS_RICH_TEXT_EXT(item);
-	get_bounds (text, px1, py1, px2, py2);
+
+	adjust_for_anchors(text, px1, px2);
+
+	*px2 = *px1 + text->_priv->width;
+	*py2 = *py1 + text->_priv->height;
 }
 
 static void
@@ -2186,23 +2195,33 @@ gnome_canvas_rich_text_ext_render(GnomeCanvasItem *item, GnomeCanvasBuf *buf)
 	GnomeCanvasRichTextExt *text = GNOME_CANVAS_RICH_TEXT_EXT (item);
 	int x, y, i, r;
 	double x0, y0, h, w;
-	guchar* data;
+	guchar *data;
 	double i2w[6], w2c[6], i2c[6];
 	double ax, ay;
 	double x1, y1, x2, y2;
 	ArtPoint ip, cp;
 	GdkPixbuf *pxb;
-	GdkPixmap* pb;
-	GdkGC* gc;
-	GdkPixbuf* pixbuf;
-	GdkColormap* map;
+	GdkPixmap *pb;
+	GdkGC *gc, *oldgc;
+	GdkPixbuf *pixbuf;
+	GdkColormap *map;
+	PangoMatrix matrix;
 
     gnome_canvas_buf_ensure_buf (buf);
+	gnome_canvas_rich_text_ext_ensure_layout (text);
 
 	gnome_canvas_item_i2w_affine(item, i2w);
 	gnome_canvas_w2c_affine(item->canvas, w2c);
 	art_affine_multiply(i2c, i2w, w2c);
 
+	matrix.xx = i2c[0];
+	matrix.xy = i2c[1];
+	matrix.yx = i2c[2];
+	matrix.yy = i2c[3];
+	matrix.x0 = 0.;
+	matrix.y0 = 0.;
+	pango_context_set_matrix (text->_priv->ltr_context, &matrix);
+	pango_context_set_matrix (text->_priv->rtl_context, &matrix);
 	adjust_for_anchors(text, &ax, &ay);
 
 	ip.x = floor(ax + 0.5);
@@ -2217,11 +2236,13 @@ gnome_canvas_rich_text_ext_render(GnomeCanvasItem *item, GnomeCanvasBuf *buf)
 	x2 = cp.x;
 	y2 = cp.y;
 
-	ip.x = text->_priv->x;
-	ip.y = text->_priv->y;
-	art_affine_point(&cp, &ip, i2c);
-	x0 = cp.x;
-	y0 = cp.y;
+//	ip.x = text->_priv->x;
+//	ip.y = text->_priv->y;
+//	art_affine_point(&cp, &ip, i2c);
+//	x0 = cp.x;
+//	y0 = cp.y;
+	x0 = x1;
+	y0 = y1;
 
 	w = x2 - x1 + 1;
 	h = y2 - y1 + 1;
@@ -2243,24 +2264,30 @@ gnome_canvas_rich_text_ext_render(GnomeCanvasItem *item, GnomeCanvasBuf *buf)
 	else y = y0 - buf->rect.y0;
 	if (y0 + h >= buf->rect.y1 + 1)
 		h = buf->rect.y1 - y0 + 1;
-	if (((int) w == 0) || ((int) h == 0)) return;
+	if (((int) w <= 0) || ((int) h <= 0)) return;
 	pxb = gdk_pixbuf_new_from_data (buf->buf + y * buf->buf_rowstride + x * 3,
 																GDK_COLORSPACE_RGB, FALSE, 8, (int) w, (int) h,
 																buf->buf_rowstride, NULL, NULL);
 	pb = gdk_pixmap_new (item->canvas->layout.bin_window, (int) w, (int) h, gtk_widget_get_visual ((GtkWidget*) item->canvas)->depth);
 	gc = gdk_gc_new (pb);
 	gdk_pixbuf_render_to_drawable (pxb, pb, gc, 0, 0, 0, 0, w, h, GDK_RGB_DITHER_NONE, 0, 0);
-	gtk_text_layout_set_screen_width(text->_priv->layout, x2 - x1);
+	gtk_text_layout_set_screen_width(text->_priv->layout, text->_priv->width);
+	oldgc = GTK_WIDGET(item->canvas)->style->text_gc[GTK_WIDGET(item->canvas)->state];
+	if (oldgc == NULL)
+		GTK_WIDGET(item->canvas)->style->text_gc[GTK_WIDGET(item->canvas)->state] = gc;
         /* FIXME: should last arg be NULL? */
 	gtk_text_layout_draw(
 		text->_priv->layout,
 		GTK_WIDGET(item->canvas),
 		pb,
 		GTK_WIDGET (item->canvas)->style->text_gc[GTK_STATE_NORMAL],
-		x0 - x1, y0 - y1,
-		0, 0, x2 - x0, y2 - y0,
+		/* The following assumes there is no rotation !*/
+		(x0 - x1) / i2c[0], (y0 - y1) / i2c[3],
+		0, 0, (x2 - x0) / i2c[0], (y2 - y0) / i2c[3],
 		NULL);
 
+	pango_context_set_matrix (text->_priv->ltr_context, NULL);
+	pango_context_set_matrix (text->_priv->rtl_context, NULL);
 	pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, (int) w, (int) h);
 	map = gdk_colormap_new (gtk_widget_get_visual ((GtkWidget*) item->canvas), FALSE);
 	gdk_pixbuf_get_from_drawable (pixbuf, pb, map, 0, 0, 0, 0, (int) w, (int) h);
@@ -2269,6 +2296,8 @@ gnome_canvas_rich_text_ext_render(GnomeCanvasItem *item, GnomeCanvasBuf *buf)
 	h--; w--;/*FIXME: be sure that h and w are the good values. This line avoids a buffer overflow*/
 	for (i = 0; i < h; i++) 
 		memcpy (buf->buf + (y + i) * buf->buf_rowstride + x * 3, data + i * r, (int) w * 3);
+	if (oldgc == NULL)
+		GTK_WIDGET(item->canvas)->style->text_gc[GTK_WIDGET(item->canvas)->state] = NULL;
 	g_object_unref (pixbuf);
 	g_object_unref (map);
 	g_object_unref (gc);
@@ -2317,7 +2346,7 @@ gnome_canvas_rich_text_ext_add_tag(GnomeCanvasRichTextExt *text, char *tag_name,
 
 PangoLayout* gnome_canvas_rich_text_ext_get_pango_layout (GnomeCanvasRichTextExt *text, gint line)
 {
-	gint top = 0, bottom, firsty;
+	gint top = 0, bottom = 0, firsty;
 	GSList* lines;
 	GtkTextLine* text_line;
 	g_return_val_if_fail (GNOME_IS_CANVAS_RICH_TEXT_EXT (text), NULL);
@@ -2424,6 +2453,7 @@ gnome_canvas_rich_text_ext_ensure_rendered (GnomeCanvasRichTextExt *text)
 	GdkPixbuf *pixbuf;
 	GnomeCanvasBuf buf;
 
+	gnome_canvas_rich_text_ext_ensure_layout (text);
 	get_bounds (text, &x1, &y1, &x2, &y2);
 	w = (int) (ceil (x2) - floor (x1)) + 1, h = (int) (ceil (y2) - floor (y1)) + 1;
 	pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, w, h);
