@@ -4,7 +4,7 @@
  * GChemPaint arrows plugin
  * arrowtool.cc 
  *
- * Copyright (C) 2001-2005 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2006 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -34,6 +34,7 @@
 #include "lib/mesomery-arrow.h"
 #include "lib/reaction-arrow.h"
 #include "lib/settings.h"
+#include "lib/theme.h"
 #include <math.h>
 #include <gconf/gconf-client.h>
 
@@ -69,75 +70,76 @@ gcpArrowToolDlg::gcpArrowToolDlg (gcpApplication* App, bool FullHeads): Dialog(A
 {
 	GtkTable *table = GTK_TABLE (glade_xml_get_widget (xml, "heads-table"));
 	GnomeCanvas *canvas = GNOME_CANVAS (gnome_canvas_new_aa ());
-	double width = (DefaultArrowLength + 2 * DefaultArrowPadding) * DefaultZoomFactor,
-		height = (DefaultArrowDist + DefaultBondWidth + 2 * (DefaultArrowHeadB + DefaultPadding)) * DefaultZoomFactor;
+	gcpTheme *Theme = ThemeManager.GetTheme ("Default");
+	double width = (Theme->GetArrowLength () + 2 * Theme->GetArrowPadding ()) * Theme->GetZoomFactor (),
+		height = (Theme->GetArrowDist () + Theme->GetArrowWidth () + 2 * (Theme->GetArrowHeadB () + Theme->GetPadding ())) * Theme->GetZoomFactor ();
 	gtk_widget_set_size_request (GTK_WIDGET (canvas), (int) width, (int) height);
 	GnomeCanvasGroup *group = gnome_canvas_root (canvas);
 	GnomeCanvasPoints *points = gnome_canvas_points_new (2);
-	gnome_canvas_set_scroll_region(canvas, 0, 0, DefaultArrowLength, DefaultArrowDist + DefaultBondWidth + 2 * DefaultArrowHeadB);
-	points->coords[0] = (width - DefaultArrowLength * DefaultZoomFactor) / 2.;
-	points->coords[1] = points->coords[3] = (height - DefaultArrowDist * DefaultZoomFactor) / 2.;
-	points->coords[2] = (width + DefaultArrowLength * DefaultZoomFactor) / 2.;
+	gnome_canvas_set_scroll_region(canvas, 0, 0, Theme ->GetArrowLength (), Theme->GetArrowDist () + Theme->GetArrowWidth () + 2 * Theme->GetArrowHeadB ());
+	points->coords[0] = (width - Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
+	points->coords[1] = points->coords[3] = (height - Theme->GetArrowDist () * Theme->GetZoomFactor ()) / 2.;
+	points->coords[2] = (width + Theme ->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
 	gnome_canvas_item_new (
 						group,
 						gnome_canvas_line_ext_get_type (),
 						"points", points,
 						"fill_color", "black",
-						"width_units", DefaultBondWidth * DefaultZoomFactor,
+						"width_units", Theme->GetArrowWidth () * Theme->GetZoomFactor (),
 						"last_arrowhead", true,
-						"arrow_shape_a",DefaultArrowHeadA * DefaultZoomFactor,
-						"arrow_shape_b",DefaultArrowHeadB * DefaultZoomFactor,
-						"arrow_shape_c",DefaultArrowHeadC * DefaultZoomFactor,
+						"arrow_shape_a",Theme->GetArrowHeadA () * Theme->GetZoomFactor (),
+						"arrow_shape_b",Theme->GetArrowHeadB () * Theme->GetZoomFactor (),
+						"arrow_shape_c",Theme->GetArrowHeadC () * Theme->GetZoomFactor (),
 						"last_arrowhead_style", (unsigned char)ARROW_HEAD_LEFT,
 						NULL);
-	points->coords[0] = (width + DefaultArrowLength * DefaultZoomFactor) / 2.;
-	points->coords[1] = points->coords[3] = (height + DefaultArrowDist * DefaultZoomFactor) / 2.;
-	points->coords[2] = (width - DefaultArrowLength * DefaultZoomFactor) / 2.;
+	points->coords[0] = (width + Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
+	points->coords[1] = points->coords[3] = (height + Theme->GetArrowDist () * Theme->GetZoomFactor ()) / 2.;
+	points->coords[2] = (width - Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
 	gnome_canvas_item_new (
 						group,
 						gnome_canvas_line_ext_get_type (),
 						"points", points,
 						"fill_color", "black",
-						"width_units",DefaultBondWidth * DefaultZoomFactor,
+						"width_units",Theme->GetArrowWidth () * Theme->GetZoomFactor (),
 						"last_arrowhead", true,
-						"arrow_shape_a",DefaultArrowHeadA * DefaultZoomFactor,
-						"arrow_shape_b",DefaultArrowHeadB * DefaultZoomFactor,
-						"arrow_shape_c",DefaultArrowHeadC * DefaultZoomFactor,
+						"arrow_shape_a",Theme->GetArrowHeadA () * Theme->GetZoomFactor (),
+						"arrow_shape_b",Theme->GetArrowHeadB () * Theme->GetZoomFactor (),
+						"arrow_shape_c",Theme->GetArrowHeadC  ()* Theme->GetZoomFactor (),
 						"last_arrowhead_style", (unsigned char)ARROW_HEAD_LEFT,
 						NULL);
 	gtk_table_attach (table, GTK_WIDGET (canvas), 1, 2, 0, 1, GTK_FILL, GTK_FILL, 10, 0);
 	canvas = GNOME_CANVAS (gnome_canvas_new_aa ());
 	gtk_widget_set_size_request (GTK_WIDGET (canvas), (int) width, (int) height);
 	group = gnome_canvas_root (canvas);
-	gnome_canvas_set_scroll_region(canvas, 0, 0, DefaultArrowLength, DefaultArrowDist + DefaultBondWidth + 2 * DefaultArrowHeadB);
-	points->coords[0] = (width - DefaultArrowLength * DefaultZoomFactor) / 2.;
-	points->coords[1] = points->coords[3] = (height - DefaultArrowDist * DefaultZoomFactor) / 2.;
-	points->coords[2] = (width + DefaultArrowLength * DefaultZoomFactor) / 2.;
+	gnome_canvas_set_scroll_region(canvas, 0, 0, Theme->GetArrowLength (), Theme->GetArrowDist () + Theme->GetArrowWidth () + 2 * Theme->GetArrowHeadB ());
+	points->coords[0] = (width - Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
+	points->coords[1] = points->coords[3] = (height - Theme->GetArrowDist () * Theme->GetZoomFactor ()) / 2.;
+	points->coords[2] = (width + Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
 	gnome_canvas_item_new (
 						group,
 						gnome_canvas_line_ext_get_type (),
 						"points", points,
 						"fill_color", "black",
-						"width_units", DefaultBondWidth * DefaultZoomFactor,
+						"width_units", Theme->GetArrowWidth () * Theme->GetZoomFactor (),
 						"last_arrowhead", true,
-						"arrow_shape_a",DefaultArrowHeadA * DefaultZoomFactor,
-						"arrow_shape_b",DefaultArrowHeadB * DefaultZoomFactor,
-						"arrow_shape_c",DefaultArrowHeadC * DefaultZoomFactor,
+						"arrow_shape_a",Theme->GetArrowHeadA () * Theme->GetZoomFactor (),
+						"arrow_shape_b",Theme->GetArrowHeadB () * Theme->GetZoomFactor (),
+						"arrow_shape_c",Theme->GetArrowHeadC () * Theme->GetZoomFactor (),
 						"last_arrowhead_style", (unsigned char)ARROW_HEAD_BOTH,
 						NULL);
-	points->coords[0] = (width + DefaultArrowLength * DefaultZoomFactor) / 2.;
-	points->coords[1] = points->coords[3] = (height + DefaultArrowDist * DefaultZoomFactor) / 2.;
-	points->coords[2] = (width - DefaultArrowLength * DefaultZoomFactor) / 2.;
+	points->coords[0] = (width + Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
+	points->coords[1] = points->coords[3] = (height + Theme->GetArrowDist () * Theme->GetZoomFactor ()) / 2.;
+	points->coords[2] = (width - Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
 	gnome_canvas_item_new (
 						group,
 						gnome_canvas_line_ext_get_type (),
 						"points", points,
 						"fill_color", "black",
-						"width_units",DefaultBondWidth * DefaultZoomFactor,
+						"width_units",Theme->GetArrowWidth () * Theme->GetZoomFactor (),
 						"last_arrowhead", true,
-						"arrow_shape_a",DefaultArrowHeadA * DefaultZoomFactor,
-						"arrow_shape_b",DefaultArrowHeadB * DefaultZoomFactor,
-						"arrow_shape_c",DefaultArrowHeadC * DefaultZoomFactor,
+						"arrow_shape_a",Theme->GetArrowHeadA () * Theme->GetZoomFactor (),
+						"arrow_shape_b",Theme->GetArrowHeadB () * Theme->GetZoomFactor (),
+						"arrow_shape_c",Theme->GetArrowHeadC () * Theme->GetZoomFactor (),
 						"last_arrowhead_style", (unsigned char)ARROW_HEAD_BOTH,
 						NULL);
 	gtk_table_attach (table, GTK_WIDGET (canvas), 1, 2, 1, 2, GTK_FILL, GTK_FILL, 10, 0);
@@ -178,8 +180,10 @@ gcpArrowTool::~gcpArrowTool()
 bool gcpArrowTool::OnClicked()
 {
 	if (m_pObject) return false;
+		gcpDocument *pDoc = m_pView->GetDoc ();
+	gcpTheme *pTheme = pDoc->GetTheme ();
 	m_y1 = m_y0;
-	m_x1 = m_x0 + m_pData->ArrowLength;
+	m_x1 = m_x0 + pTheme->GetArrowLength () * pTheme->GetZoomFactor ();
 	switch(m_ArrowType)
 	{
 		case gcpSimpleArrow:
@@ -192,17 +196,17 @@ bool gcpArrowTool::OnClicked()
 										gnome_canvas_line_ext_get_type(),
 										"points", points,
 										"fill_color", AddColor,
-										"width_units", m_pData->BondWidth,
+										"width_units", pTheme->GetArrowWidth (),
 										"last_arrowhead", true,
-										"arrow_shape_a", m_pData->ArrowHeadA,
-										"arrow_shape_b", m_pData->ArrowHeadB,
-										"arrow_shape_c", m_pData->ArrowHeadC,
+										"arrow_shape_a", pTheme->GetArrowHeadA (),
+										"arrow_shape_b", pTheme->GetArrowHeadB (),
+										"arrow_shape_c", pTheme->GetArrowHeadC (),
 										"last_arrowhead_style", (unsigned char)ARROW_HEAD_BOTH,
 										NULL);
 			break;
 		case gcpReversibleArrow:
 			points->coords[0] = m_x0;
-			points->coords[1] = points->coords[3] = m_y0 - m_pData->ArrowDist / 2;
+			points->coords[1] = points->coords[3] = m_y0 - pTheme->GetArrowDist () / 2;
 			points->coords[2] = m_x1;
 			m_pItem = gnome_canvas_item_new(m_pGroup, gnome_canvas_group_ext_get_type(), NULL);
 			gnome_canvas_item_new(
@@ -210,32 +214,32 @@ bool gcpArrowTool::OnClicked()
 								gnome_canvas_line_ext_get_type(),
 								"points", points,
 								"fill_color", AddColor,
-								"width_units", m_pData->BondWidth,
+								"width_units", pTheme->GetArrowWidth (),
 								"last_arrowhead", true,
-								"arrow_shape_a", m_pData->ArrowHeadA,
-								"arrow_shape_b", m_pData->ArrowHeadB,
-								"arrow_shape_c", m_pData->ArrowHeadC,
+								"arrow_shape_a", pTheme->GetArrowHeadA (),
+								"arrow_shape_b", pTheme->GetArrowHeadB (),
+								"arrow_shape_c", pTheme->GetArrowHeadC (),
 								"last_arrowhead_style", (unsigned char)ARROW_HEAD_LEFT,
 								NULL);
 			points->coords[2] = m_x0;
-			points->coords[1] = points->coords[3] = m_y0 + m_pData->ArrowDist / 2;
-			points->coords[0] = m_x0 + m_pData->ArrowLength;
+			points->coords[1] = points->coords[3] = m_y0 + pTheme->GetArrowDist () / 2;
+			points->coords[0] = m_x0 + pDoc->GetArrowLength ();
 			gnome_canvas_item_new(
 								GNOME_CANVAS_GROUP(m_pItem),
 								gnome_canvas_line_ext_get_type(),
 								"points", points,
 								"fill_color", AddColor,
-								"width_units", m_pData->BondWidth,
+								"width_units", pTheme->GetArrowWidth (),
 								"last_arrowhead", true,
-								"arrow_shape_a", m_pData->ArrowHeadA,
-								"arrow_shape_b", m_pData->ArrowHeadB,
-								"arrow_shape_c", m_pData->ArrowHeadC,
+								"arrow_shape_a", pTheme->GetArrowHeadA (),
+								"arrow_shape_b", pTheme->GetArrowHeadB (),
+								"arrow_shape_c", pTheme->GetArrowHeadC (),
 								"last_arrowhead_style", (unsigned char)ARROW_HEAD_LEFT,
 								NULL);
 			break;
 		case gcpFullReversibleArrow:
 			points->coords[0] = m_x0;
-			points->coords[1] = points->coords[3] = m_y0 - m_pData->ArrowDist / 2;
+			points->coords[1] = points->coords[3] = m_y0 - pTheme->GetArrowDist () / 2;
 			points->coords[2] = m_x1;
 			m_pItem = gnome_canvas_item_new(m_pGroup, gnome_canvas_group_ext_get_type(), NULL);
 			gnome_canvas_item_new(
@@ -243,26 +247,26 @@ bool gcpArrowTool::OnClicked()
 								gnome_canvas_line_ext_get_type(),
 								"points", points,
 								"fill_color", AddColor,
-								"width_units", m_pData->BondWidth,
+								"width_units", pTheme->GetArrowWidth (),
 								"last_arrowhead", true,
-								"arrow_shape_a", m_pData->ArrowHeadA,
-								"arrow_shape_b", m_pData->ArrowHeadB,
-								"arrow_shape_c", m_pData->ArrowHeadC,
+								"arrow_shape_a", pTheme->GetArrowHeadA (),
+								"arrow_shape_b", pTheme->GetArrowHeadB (),
+								"arrow_shape_c", pTheme->GetArrowHeadC (),
 								"last_arrowhead_style", (unsigned char)ARROW_HEAD_BOTH,
 								NULL);
 			points->coords[2] = m_x0;
-			points->coords[1] = points->coords[3] = m_y0 + m_pData->ArrowDist / 2;
-			points->coords[0] = m_x0 + m_pData->ArrowLength;
+			points->coords[1] = points->coords[3] = m_y0 + pTheme->GetArrowDist () / 2;
+			points->coords[0] = m_x0 + pDoc->GetArrowLength ();
 			gnome_canvas_item_new(
 								GNOME_CANVAS_GROUP(m_pItem),
 								gnome_canvas_line_ext_get_type(),
 								"points", points,
 								"fill_color", AddColor,
-								"width_units", m_pData->BondWidth,
+								"width_units", pTheme->GetArrowWidth (),
 								"last_arrowhead", true,
-								"arrow_shape_a", m_pData->ArrowHeadA,
-								"arrow_shape_b", m_pData->ArrowHeadB,
-								"arrow_shape_c", m_pData->ArrowHeadC,
+								"arrow_shape_a", pTheme->GetArrowHeadA (),
+								"arrow_shape_b", pTheme->GetArrowHeadB (),
+								"arrow_shape_c", pTheme->GetArrowHeadC (),
 								"last_arrowhead_style", (unsigned char)ARROW_HEAD_BOTH,
 								NULL);
 			break;
@@ -276,31 +280,31 @@ bool gcpArrowTool::OnClicked()
 										gnome_canvas_line_ext_get_type(),
 										"points", points,
 										"fill_color", AddColor,
-										"width_units", m_pData->BondWidth,
+										"width_units", pTheme->GetArrowWidth (),
 										"first_arrowhead", true,
 										"last_arrowhead", true,
-										"arrow_shape_a", m_pData->ArrowHeadA,
-										"arrow_shape_b", m_pData->ArrowHeadB,
-										"arrow_shape_c", m_pData->ArrowHeadC,
+										"arrow_shape_a", pTheme->GetArrowHeadA (),
+										"arrow_shape_b", pTheme->GetArrowHeadB (),
+										"arrow_shape_c", pTheme->GetArrowHeadC (),
 										"first_arrowhead_style", (unsigned char)ARROW_HEAD_BOTH,
 										"last_arrowhead_style", (unsigned char)ARROW_HEAD_BOTH,
 										NULL);
 			break;
 		case gcpDoubleQueuedArrow: {
 			GnomeCanvasPathDef* path = gnome_canvas_path_def_new ();
-			gnome_canvas_path_def_moveto (path, m_x0, m_y0 - m_pData->ArrowDist / 2.);
-			gnome_canvas_path_def_lineto (path, m_x1 - m_pData->ArrowDist / 2., m_y0 - m_pData->ArrowDist / 2.);
-			gnome_canvas_path_def_moveto (path, m_x0, m_y0 + m_pData->ArrowDist / 2.);
-			gnome_canvas_path_def_lineto (path, m_x1 - m_pData->ArrowDist / 2., m_y0 + m_pData->ArrowDist / 2.);
-			gnome_canvas_path_def_moveto (path, m_x1 - m_pData->ArrowDist / 2. - m_pData->ArrowHeadC, m_y0 - m_pData->ArrowDist / 2. - m_pData->ArrowHeadC);
+			gnome_canvas_path_def_moveto (path, m_x0, m_y0 - pTheme->GetArrowDist () / 2.);
+			gnome_canvas_path_def_lineto (path, m_x1 - pTheme->GetArrowDist () / 2., m_y0 - pTheme->GetArrowDist () / 2.);
+			gnome_canvas_path_def_moveto (path, m_x0, m_y0 + pTheme->GetArrowDist () / 2.);
+			gnome_canvas_path_def_lineto (path, m_x1 - pTheme->GetArrowDist () / 2., m_y0 + pTheme->GetArrowDist () / 2.);
+			gnome_canvas_path_def_moveto (path, m_x1 - pTheme->GetArrowDist () / 2. - pTheme->GetArrowHeadC (), m_y0 - pTheme->GetArrowDist () / 2. - pTheme->GetArrowHeadC ());
 			gnome_canvas_path_def_lineto (path, m_x1, m_y0);
-			gnome_canvas_path_def_lineto (path, m_x1 - m_pData->ArrowDist / 2. - m_pData->ArrowHeadC, m_y0 + m_pData->ArrowDist / 2. + m_pData->ArrowHeadC);
+			gnome_canvas_path_def_lineto (path, m_x1 - pTheme->GetArrowDist () / 2. - pTheme->GetArrowHeadC (), m_y0 + pTheme->GetArrowDist () / 2. + pTheme->GetArrowHeadC ());
 			m_pItem = gnome_canvas_item_new(
 									m_pGroup,
 									gnome_canvas_bpath_ext_get_type(),
 									"bpath", path,
 									"outline_color", AddColor,
-									"width_units", m_pData->BondWidth,
+									"width_units", pTheme->GetArrowWidth (),
 									"cap-style", GDK_CAP_BUTT,
 									"join-style", GDK_JOIN_MITER,
 									NULL);
@@ -313,6 +317,8 @@ bool gcpArrowTool::OnClicked()
 void gcpArrowTool::OnDrag()
 {
 	double x1, y1, x2, y2;
+	gcpDocument *pDoc = m_pView->GetDoc ();
+	gcpTheme *pTheme = pDoc->GetTheme ();
 	if (m_pItem)
 	{
 		gnome_canvas_item_get_bounds(GNOME_CANVAS_ITEM(m_pItem), &x1, &y1, &x2, &y2);
@@ -335,7 +341,7 @@ void gcpArrowTool::OnDrag()
 		if (m_x < 0) dAngle += 180;
 	}
 	m_dAngle = dAngle * M_PI / 180;
-	double d = (m_nState & GDK_SHIFT_MASK)? sqrt(square(m_x) + square(m_y)): m_pData->ArrowLength;
+	double d = (m_nState & GDK_SHIFT_MASK)? sqrt(square(m_x) + square(m_y)): pDoc->GetArrowLength () * pTheme->GetZoomFactor ();
 	char tmp[32];
 	if (dAngle < 0) dAngle += 360;
 	snprintf(tmp, sizeof(tmp) - 1, _("Orientation: %g"), dAngle);
@@ -352,81 +358,81 @@ void gcpArrowTool::OnDrag()
 										gnome_canvas_line_ext_get_type(),
 										"points", points,
 										"fill_color", AddColor,
-										"width_units", m_pData->BondWidth,
+										"width_units", pTheme->GetArrowWidth (),
 										"last_arrowhead", true,
-										"arrow_shape_a", m_pData->ArrowHeadA,
-										"arrow_shape_b", m_pData->ArrowHeadB,
-										"arrow_shape_c", m_pData->ArrowHeadC,
+										"arrow_shape_a", pTheme->GetArrowHeadA (),
+										"arrow_shape_b", pTheme->GetArrowHeadB (),
+										"arrow_shape_c", pTheme->GetArrowHeadC (),
 										"last_arrowhead_style", (unsigned char)ARROW_HEAD_BOTH,
 										NULL);
 			break;
 		case gcpReversibleArrow:
-			points->coords[0] = m_x0 - m_pData->ArrowDist / 2 * sin(m_dAngle);
-			points->coords[1] = m_y0 - m_pData->ArrowDist / 2 * cos(m_dAngle);
-			points->coords[2] = m_x1 - m_pData->ArrowDist / 2 * sin(m_dAngle);
-			points->coords[3] = m_y1 - m_pData->ArrowDist / 2 * cos(m_dAngle);
+			points->coords[0] = m_x0 - pTheme->GetArrowDist () / 2 * sin(m_dAngle);
+			points->coords[1] = m_y0 - pTheme->GetArrowDist () / 2 * cos(m_dAngle);
+			points->coords[2] = m_x1 - pTheme->GetArrowDist () / 2 * sin(m_dAngle);
+			points->coords[3] = m_y1 - pTheme->GetArrowDist () / 2 * cos(m_dAngle);
 			m_pItem = gnome_canvas_item_new(m_pGroup, gnome_canvas_group_ext_get_type(), NULL);
 			gnome_canvas_item_new(
 								GNOME_CANVAS_GROUP(m_pItem),
 								gnome_canvas_line_ext_get_type(),
 								"points", points,
 								"fill_color", AddColor,
-								"width_units", m_pData->BondWidth,
+								"width_units", pTheme->GetArrowWidth (),
 								"last_arrowhead", true,
-								"arrow_shape_a", m_pData->ArrowHeadA,
-								"arrow_shape_b", m_pData->ArrowHeadB,
-								"arrow_shape_c", m_pData->ArrowHeadC,
+								"arrow_shape_a", pTheme->GetArrowHeadA (),
+								"arrow_shape_b", pTheme->GetArrowHeadB (),
+								"arrow_shape_c", pTheme->GetArrowHeadC (),
 								"last_arrowhead_style", (unsigned char)ARROW_HEAD_LEFT,
 								NULL);
-			points->coords[2] = m_x0 + m_pData->ArrowDist / 2 * sin(m_dAngle);
-			points->coords[3] = m_y0 + m_pData->ArrowDist / 2 * cos(m_dAngle);
-			points->coords[0] = m_x1 + m_pData->ArrowDist / 2 * sin(m_dAngle);
-			points->coords[1] = m_y1 + m_pData->ArrowDist / 2 * cos(m_dAngle);
+			points->coords[2] = m_x0 + pTheme->GetArrowDist () / 2 * sin(m_dAngle);
+			points->coords[3] = m_y0 + pTheme->GetArrowDist () / 2 * cos(m_dAngle);
+			points->coords[0] = m_x1 + pTheme->GetArrowDist () / 2 * sin(m_dAngle);
+			points->coords[1] = m_y1 + pTheme->GetArrowDist () / 2 * cos(m_dAngle);
 			gnome_canvas_item_new(
 								GNOME_CANVAS_GROUP(m_pItem),
 								gnome_canvas_line_ext_get_type(),
 								"points", points,
 								"fill_color", AddColor,
-								"width_units", m_pData->BondWidth,
+								"width_units", pTheme->GetArrowWidth (),
 								"last_arrowhead", true,
-								"arrow_shape_a", m_pData->ArrowHeadA,
-								"arrow_shape_b", m_pData->ArrowHeadB,
-								"arrow_shape_c", m_pData->ArrowHeadC,
+								"arrow_shape_a", pTheme->GetArrowHeadA (),
+								"arrow_shape_b", pTheme->GetArrowHeadB (),
+								"arrow_shape_c", pTheme->GetArrowHeadC (),
 								"last_arrowhead_style", (unsigned char)ARROW_HEAD_LEFT,
 								NULL);
 			break;
 		case gcpFullReversibleArrow:
-			points->coords[0] = m_x0 - m_pData->ArrowDist / 2 * sin(m_dAngle);
-			points->coords[1] = m_y0 - m_pData->ArrowDist / 2 * cos(m_dAngle);
-			points->coords[2] = m_x1 - m_pData->ArrowDist / 2 * sin(m_dAngle);
-			points->coords[3] = m_y1 - m_pData->ArrowDist / 2 * cos(m_dAngle);
+			points->coords[0] = m_x0 - pTheme->GetArrowDist () / 2 * sin(m_dAngle);
+			points->coords[1] = m_y0 - pTheme->GetArrowDist () / 2 * cos(m_dAngle);
+			points->coords[2] = m_x1 - pTheme->GetArrowDist () / 2 * sin(m_dAngle);
+			points->coords[3] = m_y1 - pTheme->GetArrowDist () / 2 * cos(m_dAngle);
 			m_pItem = gnome_canvas_item_new(m_pGroup, gnome_canvas_group_ext_get_type(), NULL);
 			gnome_canvas_item_new(
 								GNOME_CANVAS_GROUP(m_pItem),
 								gnome_canvas_line_ext_get_type(),
 								"points", points,
 								"fill_color", AddColor,
-								"width_units", m_pData->BondWidth,
+								"width_units", pTheme->GetArrowWidth (),
 								"last_arrowhead", true,
-								"arrow_shape_a", m_pData->ArrowHeadA,
-								"arrow_shape_b", m_pData->ArrowHeadB,
-								"arrow_shape_c", m_pData->ArrowHeadC,
+								"arrow_shape_a", pTheme->GetArrowHeadA (),
+								"arrow_shape_b", pTheme->GetArrowHeadB (),
+								"arrow_shape_c", pTheme->GetArrowHeadC (),
 								"last_arrowhead_style", (unsigned char)ARROW_HEAD_BOTH,
 								NULL);
-			points->coords[2] = m_x0 + m_pData->ArrowDist / 2 * sin(m_dAngle);
-			points->coords[3] = m_y0 + m_pData->ArrowDist / 2 * cos(m_dAngle);
-			points->coords[0] = m_x1 + m_pData->ArrowDist / 2 * sin(m_dAngle);
-			points->coords[1] = m_y1 + m_pData->ArrowDist / 2 * cos(m_dAngle);
+			points->coords[2] = m_x0 + pTheme->GetArrowDist () / 2 * sin(m_dAngle);
+			points->coords[3] = m_y0 + pTheme->GetArrowDist () / 2 * cos(m_dAngle);
+			points->coords[0] = m_x1 + pTheme->GetArrowDist () / 2 * sin(m_dAngle);
+			points->coords[1] = m_y1 + pTheme->GetArrowDist () / 2 * cos(m_dAngle);
 			gnome_canvas_item_new(
 								GNOME_CANVAS_GROUP(m_pItem),
 								gnome_canvas_line_ext_get_type(),
 								"points", points,
 								"fill_color", AddColor,
-								"width_units", m_pData->BondWidth,
+								"width_units", pTheme->GetArrowWidth (),
 								"last_arrowhead", true,
-								"arrow_shape_a", m_pData->ArrowHeadA,
-								"arrow_shape_b", m_pData->ArrowHeadB,
-								"arrow_shape_c", m_pData->ArrowHeadC,
+								"arrow_shape_a", pTheme->GetArrowHeadA (),
+								"arrow_shape_b", pTheme->GetArrowHeadB (),
+								"arrow_shape_c", pTheme->GetArrowHeadC (),
 								"last_arrowhead_style", (unsigned char)ARROW_HEAD_BOTH,
 								NULL);
 			break;
@@ -438,27 +444,27 @@ void gcpArrowTool::OnDrag()
 										gnome_canvas_line_ext_get_type(),
 										"points", points,
 										"fill_color", AddColor,
-										"width_units", m_pData->BondWidth,
+										"width_units", pTheme->GetArrowWidth (),
 										"first_arrowhead", true,
 										"last_arrowhead", true,
-										"arrow_shape_a", m_pData->ArrowHeadA,
-										"arrow_shape_b", m_pData->ArrowHeadB,
-										"arrow_shape_c", m_pData->ArrowHeadC,
+										"arrow_shape_a", pTheme->GetArrowHeadA (),
+										"arrow_shape_b", pTheme->GetArrowHeadB (),
+										"arrow_shape_c", pTheme->GetArrowHeadC (),
 										"first_arrowhead_style", (unsigned char)ARROW_HEAD_BOTH,
 										"last_arrowhead_style", (unsigned char)ARROW_HEAD_BOTH,
 										NULL);
 			break;
 		case gcpDoubleQueuedArrow: {
 			double x1, y1;
-			x1 = m_pData->ArrowDist / 2 * sin (m_dAngle);
-			y1 = m_pData->ArrowDist / 2 * cos (m_dAngle);
+			x1 = pTheme->GetArrowDist () / 2 * sin (m_dAngle);
+			y1 = pTheme->GetArrowDist () / 2 * cos (m_dAngle);
 			GnomeCanvasPathDef* path = gnome_canvas_path_def_new ();
 			gnome_canvas_path_def_moveto (path, m_x0 - x1, m_y0 - y1);
 			gnome_canvas_path_def_lineto (path, m_x1 - x1 - y1, m_y1 - y1 + x1);
 			gnome_canvas_path_def_moveto (path, m_x0 + x1, m_y0 + y1);
 			gnome_canvas_path_def_lineto (path, m_x1 + x1 - y1, m_y1 + y1 + x1);
-			x1 += m_pData->ArrowHeadC * sin (m_dAngle);
-			y1 += m_pData->ArrowHeadC * cos (m_dAngle);
+			x1 += pTheme->GetArrowHeadC () * sin (m_dAngle);
+			y1 += pTheme->GetArrowHeadC () * cos (m_dAngle);
 			gnome_canvas_path_def_moveto (path, m_x1 - x1 - y1, m_y1 - y1 + x1);
 			gnome_canvas_path_def_lineto (path, m_x1, m_y1);
 			gnome_canvas_path_def_lineto (path, m_x1 + x1 - y1, m_y1 + y1 + x1);
@@ -467,7 +473,7 @@ void gcpArrowTool::OnDrag()
 									gnome_canvas_bpath_ext_get_type(),
 									"bpath", path,
 									"outline_color", AddColor,
-									"width_units", m_pData->BondWidth,
+									"width_units", pTheme->GetArrowWidth (),
 									"cap-style", GDK_CAP_BUTT,
 									"join-style", GDK_JOIN_MITER,
 									NULL);
@@ -539,40 +545,41 @@ GtkWidget *gcpArrowTool::GetPropertyPage ()
 	GladeXML *xml = glade_xml_new (DATADIR"/gchempaint-unstable/ui/arrowtool.glade", "heads-box", GETTEXT_PACKAGE);
 	GtkTable *table = GTK_TABLE (glade_xml_get_widget (xml, "heads-table"));
 	GnomeCanvas *canvas = GNOME_CANVAS (gnome_canvas_new_aa ());
-	double width = (DefaultArrowLength + 2 * DefaultArrowPadding) * DefaultZoomFactor,
-		height = (DefaultArrowDist + DefaultBondWidth + 2 * (DefaultArrowHeadB + DefaultPadding)) * DefaultZoomFactor;
+	gcpTheme *Theme = ThemeManager.GetTheme ("Default");
+	double width = (Theme->GetArrowLength () + 2 * Theme->GetArrowPadding ()) * Theme->GetZoomFactor (),
+		height = (Theme->GetArrowDist () + Theme->GetArrowWidth () + 2 * (Theme->GetArrowHeadB () + Theme->GetPadding ())) * Theme->GetZoomFactor ();
 	gtk_widget_set_size_request (GTK_WIDGET (canvas), (int) width, (int) height);
 	GnomeCanvasGroup *group = gnome_canvas_root (canvas);
 	GnomeCanvasPoints *points = gnome_canvas_points_new (2);
-	gnome_canvas_set_scroll_region (canvas, 0, 0, DefaultArrowLength, DefaultArrowDist + DefaultBondWidth + 2 * DefaultArrowHeadB);
-	points->coords[0] = (width - DefaultArrowLength * DefaultZoomFactor) / 2.;
-	points->coords[1] = points->coords[3] = (height - DefaultArrowDist * DefaultZoomFactor) / 2.;
-	points->coords[2] = (width + DefaultArrowLength * DefaultZoomFactor) / 2.;
+	gnome_canvas_set_scroll_region (canvas, 0, 0, Theme->GetArrowLength (), Theme->GetArrowDist () + Theme->GetArrowWidth () + 2 * Theme->GetArrowHeadB ());
+	points->coords[0] = (width - Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
+	points->coords[1] = points->coords[3] = (height - Theme->GetArrowDist  ()* Theme->GetZoomFactor ()) / 2.;
+	points->coords[2] = (width + Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
 	gnome_canvas_item_new (
 						group,
 						gnome_canvas_line_ext_get_type (),
 						"points", points,
 						"fill_color", "black",
-						"width_units", DefaultBondWidth * DefaultZoomFactor,
+						"width_units", Theme->GetArrowWidth () * Theme->GetZoomFactor (),
 						"last_arrowhead", true,
-						"arrow_shape_a",DefaultArrowHeadA * DefaultZoomFactor,
-						"arrow_shape_b",DefaultArrowHeadB * DefaultZoomFactor,
-						"arrow_shape_c",DefaultArrowHeadC * DefaultZoomFactor,
+						"arrow_shape_a",Theme->GetArrowHeadA () * Theme->GetZoomFactor (),
+						"arrow_shape_b",Theme->GetArrowHeadB () * Theme->GetZoomFactor (),
+						"arrow_shape_c",Theme->GetArrowHeadC () * Theme->GetZoomFactor (),
 						"last_arrowhead_style", (unsigned char)ARROW_HEAD_LEFT,
 						NULL);
-	points->coords[0] = (width + DefaultArrowLength * DefaultZoomFactor) / 2.;
-	points->coords[1] = points->coords[3] = (height + DefaultArrowDist * DefaultZoomFactor) / 2.;
-	points->coords[2] = (width - DefaultArrowLength * DefaultZoomFactor) / 2.;
+	points->coords[0] = (width + Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
+	points->coords[1] = points->coords[3] = (height + Theme->GetArrowDist () * Theme->GetZoomFactor ()) / 2.;
+	points->coords[2] = (width - Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
 	gnome_canvas_item_new (
 						group,
 						gnome_canvas_line_ext_get_type (),
 						"points", points,
 						"fill_color", "black",
-						"width_units",DefaultBondWidth * DefaultZoomFactor,
+						"width_units",Theme->GetArrowWidth () * Theme->GetZoomFactor (),
 						"last_arrowhead", true,
-						"arrow_shape_a",DefaultArrowHeadA * DefaultZoomFactor,
-						"arrow_shape_b",DefaultArrowHeadB * DefaultZoomFactor,
-						"arrow_shape_c",DefaultArrowHeadC * DefaultZoomFactor,
+						"arrow_shape_a",Theme->GetArrowHeadA () * Theme->GetZoomFactor (),
+						"arrow_shape_b",Theme->GetArrowHeadB () * Theme->GetZoomFactor (),
+						"arrow_shape_c",Theme->GetArrowHeadC () * Theme->GetZoomFactor (),
 						"last_arrowhead_style", (unsigned char)ARROW_HEAD_LEFT,
 						NULL);
 	gtk_widget_show (GTK_WIDGET (canvas));
@@ -580,35 +587,35 @@ GtkWidget *gcpArrowTool::GetPropertyPage ()
 	canvas = GNOME_CANVAS (gnome_canvas_new_aa ());
 	gtk_widget_set_size_request (GTK_WIDGET (canvas), (int) width, (int) height);
 	group = gnome_canvas_root (canvas);
-	gnome_canvas_set_scroll_region (canvas, 0, 0, DefaultArrowLength, DefaultArrowDist + DefaultBondWidth + 2 * DefaultArrowHeadB);
-	points->coords[0] = (width - DefaultArrowLength * DefaultZoomFactor) / 2.;
-	points->coords[1] = points->coords[3] = (height - DefaultArrowDist * DefaultZoomFactor) / 2.;
-	points->coords[2] = (width + DefaultArrowLength * DefaultZoomFactor) / 2.;
+	gnome_canvas_set_scroll_region (canvas, 0, 0, Theme->GetArrowLength (), Theme->GetArrowDist () + Theme->GetArrowWidth () + 2 * Theme->GetArrowHeadB ());
+	points->coords[0] = (width - Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
+	points->coords[1] = points->coords[3] = (height - Theme->GetArrowDist () * Theme->GetZoomFactor ()) / 2.;
+	points->coords[2] = (width + Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
 	gnome_canvas_item_new (
 						group,
 						gnome_canvas_line_ext_get_type (),
 						"points", points,
 						"fill_color", "black",
-						"width_units", DefaultBondWidth * DefaultZoomFactor,
+						"width_units", Theme->GetArrowWidth () * Theme->GetZoomFactor (),
 						"last_arrowhead", true,
-						"arrow_shape_a",DefaultArrowHeadA * DefaultZoomFactor,
-						"arrow_shape_b",DefaultArrowHeadB * DefaultZoomFactor,
-						"arrow_shape_c",DefaultArrowHeadC * DefaultZoomFactor,
+						"arrow_shape_a",Theme->GetArrowHeadA () * Theme->GetZoomFactor (),
+						"arrow_shape_b",Theme->GetArrowHeadB () * Theme->GetZoomFactor (),
+						"arrow_shape_c",Theme->GetArrowHeadC () * Theme->GetZoomFactor (),
 						"last_arrowhead_style", (unsigned char)ARROW_HEAD_BOTH,
 						NULL);
-	points->coords[0] = (width + DefaultArrowLength * DefaultZoomFactor) / 2.;
-	points->coords[1] = points->coords[3] = (height + DefaultArrowDist * DefaultZoomFactor) / 2.;
-	points->coords[2] = (width - DefaultArrowLength * DefaultZoomFactor) / 2.;
+	points->coords[0] = (width + Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
+	points->coords[1] = points->coords[3] = (height + Theme->GetArrowDist () * Theme->GetZoomFactor ()) / 2.;
+	points->coords[2] = (width - Theme->GetArrowLength () * Theme->GetZoomFactor ()) / 2.;
 	gnome_canvas_item_new (
 						group,
 						gnome_canvas_line_ext_get_type (),
 						"points", points,
 						"fill_color", "black",
-						"width_units",DefaultBondWidth * DefaultZoomFactor,
+						"width_units",Theme->GetArrowWidth () * Theme->GetZoomFactor (),
 						"last_arrowhead", true,
-						"arrow_shape_a",DefaultArrowHeadA * DefaultZoomFactor,
-						"arrow_shape_b",DefaultArrowHeadB * DefaultZoomFactor,
-						"arrow_shape_c",DefaultArrowHeadC * DefaultZoomFactor,
+						"arrow_shape_a",Theme->GetArrowHeadA () * Theme->GetZoomFactor (),
+						"arrow_shape_b",Theme->GetArrowHeadB () * Theme->GetZoomFactor (),
+						"arrow_shape_c",Theme->GetArrowHeadC () * Theme->GetZoomFactor (),
 						"last_arrowhead_style", (unsigned char)ARROW_HEAD_BOTH,
 						NULL);
 	gtk_widget_show (GTK_WIDGET (canvas));
