@@ -27,6 +27,7 @@
 #include "lib/settings.h"
 #include "lib/document.h"
 #include "lib/application.h"
+#include "lib/theme.h"
 #include "libgcpcanvas/gcp-canvas-group.h"
 #include "libgcpcanvas/gcp-canvas-pango.h"
 #include <math.h>
@@ -47,6 +48,7 @@ bool gcpChargeTool::OnClicked()
 	if (!m_pObject || (m_pObject->GetType () != AtomType))
 		return false;
 	gcpAtom *pAtom = (gcpAtom*) m_pObject;
+	gcpTheme *Theme = m_pView->GetDoc ()->GetTheme ();
 	m_Charge = pAtom->GetCharge () + ((GetName() == string("ChargePlus"))? 1: -1);
 	if (!pAtom->AcceptCharge (m_Charge))
 		return false;
@@ -129,33 +131,33 @@ bool gcpChargeTool::OnClicked()
 			pango_layout_get_extents (pl, NULL, &rect);
 			m_ChargeWidth = rect.width / PANGO_SCALE;
 			pango_layout_set_font_description (pl, m_pView->GetPangoSmallFontDesc ());
-			m_ChargeTWidth = m_ChargeWidth + 1. + ChargeSignSize;
+			m_ChargeTWidth = m_ChargeWidth + 1. + Theme->GetChargeSignSize ();
 		} else {
 			m_ChargeWidth = 0.;
-			m_ChargeTWidth = ChargeSignSize;
+			m_ChargeTWidth = Theme->GetChargeSignSize ();
 		}
 		switch (align) {
 		case -2:
-			xc = x + m_ChargeTWidth / 2. - ChargeSignSize;
-			y += ChargeSignHeight / 2.;
+			xc = x + m_ChargeTWidth / 2. - Theme->GetChargeSignSize ();
+			y += Theme->GetChargeSignHeight () / 2.;
 			break;
 		case -1:
-			xc = x - ChargeSignSize -1.;
+			xc = x - Theme->GetChargeSignSize () -1.;
 			break;
 		case 0:
 		case -3:
-			xc = x + m_ChargeTWidth / 2. - ChargeSignSize;
+			xc = x + m_ChargeTWidth / 2. - Theme->GetChargeSignSize ();
 			break;
 		case 1:
 			xc = x + m_ChargeWidth + 1.;
 			break;
 		case 2:
-			xc = x + m_ChargeTWidth / 2. - ChargeSignSize;
-			y -= ChargeSignHeight / 2.;
+			xc = x + m_ChargeTWidth / 2. - Theme->GetChargeSignSize ();
+			y -= Theme->GetChargeSignHeight () / 2.;
 			break;
 		}
 		x = xc - 1.;
-		yc = y - ChargeSignHeight / 2. + ChargeYAlign - ChargeSignSize + 1.;
+		yc = y - Theme->GetChargeSignHeight () / 2. + Theme->GetChargeYAlign () - Theme->GetChargeSignSize () + 1.;
 		m_x1 = x;
 		m_y1 = y;
 		m_pItem = gnome_canvas_item_new (
@@ -178,8 +180,8 @@ bool gcpChargeTool::OnClicked()
 					gnome_canvas_ellipse_get_type (),
 					"x1", xc,
 					"y1", yc,
-					"x2", xc + ChargeSignSize,
-					"y2", yc + ChargeSignSize,
+					"x2", xc + Theme->GetChargeSignSize (),
+					"y2", yc + Theme->GetChargeSignSize (),
 					"outline_color", AddColor,
 					"width_units", 0.5,
 					NULL
@@ -188,14 +190,14 @@ bool gcpChargeTool::OnClicked()
 		path[0].code = ART_MOVETO_OPEN;
 		path[0].x3 = xc + 1.;
 		path[1].code = ART_LINETO;
-		path[1].x3 = xc + ChargeSignSize - 1.;
-		path[0].y3 = path[1].y3 = yc + ChargeSignSize / 2.;
+		path[1].x3 = xc + Theme->GetChargeSignSize () - 1.;
+		path[0].y3 = path[1].y3 = yc + Theme->GetChargeSignSize () / 2.;
 		if (m_Charge > 0) {
 			path[2].code = ART_MOVETO_OPEN;
 			path[2].y3 = yc + 1.;
 			path[3].code = ART_LINETO;
-			path[3].y3 = yc + ChargeSignSize - 1.;
-			path[2].x3 = path[3].x3 = xc + ChargeSignSize / 2.;
+			path[3].y3 = yc + Theme->GetChargeSignSize () - 1.;
+			path[2].x3 = path[3].x3 = xc + Theme->GetChargeSignSize () / 2.;
 			path[4].code = ART_END;
 		} else
 			path[2].code = ART_END;
@@ -327,10 +329,11 @@ void gcpChargeTool::OnDrag()
 	} else {
 		double x, y;
 		gcpAtom *pAtom = (gcpAtom*)m_pObject;
+		gcpTheme *Theme = m_pView->GetDoc ()->GetTheme ();
 		if (!(m_nState & GDK_SHIFT_MASK) && (m_dDist >= m_dDistMax) && m_bChanged) {
 			gnome_canvas_item_hide (m_pItem);
 			m_bChanged = false;
-		} else if (align = pAtom->GetChargePosition (m_Pos, Angle * 180. / M_PI, x, y)) {
+		} else if ((align = pAtom->GetChargePosition (m_Pos, Angle * 180. / M_PI, x, y))) {
 			m_dAngle = Angle;
 			if (m_nState & GDK_SHIFT_MASK) {
 				align = 0;
@@ -342,21 +345,21 @@ void gcpChargeTool::OnDrag()
 			}
 			switch (align) {
 			case -2:
-				x += m_ChargeTWidth / 2. - ChargeSignSize - 1.;
-				y += ChargeSignHeight / 2.;
+				x += m_ChargeTWidth / 2. - Theme->GetChargeSignSize () - 1.;
+				y += Theme->GetChargeSignHeight () / 2.;
 				break;
 			case -1:
-				x-= ChargeSignSize + 2.;
+				x-= Theme->GetChargeSignSize () + 2.;
 				break;
 			case -3:
-				x += m_ChargeTWidth / 2. - ChargeSignSize - 1.;
+				x += m_ChargeTWidth / 2. - Theme->GetChargeSignSize () - 1.;
 				break;
 			case 1:
 				x += m_ChargeWidth + 2.;
 				break;
 			case 2:
-				x += m_ChargeTWidth / 2. - ChargeSignSize - 1.;
-				y -= ChargeSignHeight / 2.;
+				x += m_ChargeTWidth / 2. - Theme->GetChargeSignSize () - 1.;
+				y -= Theme->GetChargeSignHeight () / 2.;
 				break;
 			}
 			gnome_canvas_item_move (m_pItem, x - m_x1, y - m_y1);
