@@ -4,7 +4,7 @@
  * GChemPaint cycles plugin
  * cycletool.cc 
  *
- * Copyright (C) 2001-2005 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2006 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -28,6 +28,7 @@
 #include "lib/mendeleiev.h"
 #include "lib/document.h"
 #include "lib/application.h"
+#include "lib/theme.h"
 #include <math.h>
 #include <list>
 #include <vector>
@@ -68,7 +69,9 @@ bool gcpCycleTool::OnClicked()
 	double a0, a1, a2, b1, b2, m1, m2;
 	gcpAtom* pAtom, *pAtom1;
 	gcpBond* pBond, *pBond1;
-	m_dLength = DefaultBondLength * m_dZoomFactor;
+	gcpDocument *pDoc = m_pView->GetDoc ();
+	gcpTheme *pTheme = pDoc->GetTheme ();
+	m_dLength = pDoc->GetBondLength () * m_dZoomFactor;
 	map<Atom*, Bond*>::iterator j;
 	int i;
 	bool bDone = false;
@@ -220,7 +223,7 @@ bool gcpCycleTool::OnClicked()
 						pBond = (gcpBond*)pAtom->GetFirstBond(j);
 						orientations.push_back(pBond->GetAngle2DRad(pAtom));
 						unsigned k;
-						while (pBond = (gcpBond*)pAtom->GetNextBond(j))
+						while ((pBond = (gcpBond*)pAtom->GetNextBond(j)))
 						{
 							k = 0;
 							a0 = pBond->GetAngle2DRad(pAtom);
@@ -276,7 +279,7 @@ bool gcpCycleTool::OnClicked()
 								gnome_canvas_polygon_get_type(),
 								"points", points,
 								"outline_color", (m_bAllowed)? AddColor: DeleteColor,
-								"width_units", m_pData->BondWidth,
+								"width_units", pTheme->GetBondWidth (),
 								NULL);
 	return true;
 }
@@ -288,6 +291,8 @@ void gcpCycleTool::OnDrag()
 	double x1, y1, x2, y2;
 	bool bDone = false;
 	GnomeCanvasItem* pItem = gnome_canvas_get_item_at(GNOME_CANVAS(m_pWidget), m_x, m_y);
+	gcpDocument *pDoc = m_pView->GetDoc ();
+	gcpTheme *pTheme = pDoc->GetTheme ();
 	if (pItem == (GnomeCanvasItem*)m_pBackground) pItem = NULL;
 	Object* pObject = NULL;
 	if (pItem) pObject = (Object*)g_object_get_data(G_OBJECT(pItem), "object");
@@ -295,7 +300,7 @@ void gcpCycleTool::OnDrag()
 	{
 		if (m_pObject->GetType() == BondType)
 		{
-			if (((gcpBond*)m_pObject)->GetDist(m_x / m_dZoomFactor, m_y / m_dZoomFactor) < (DefaultPadding + DefaultBondWidth / 2) * m_dZoomFactor)
+			if (((gcpBond*)m_pObject)->GetDist(m_x / m_dZoomFactor, m_y / m_dZoomFactor) < (pTheme->GetPadding () + pTheme->GetBondWidth () / 2) * m_dZoomFactor)
 			{
 				if (m_pItem)
 				{
@@ -359,7 +364,7 @@ void gcpCycleTool::OnDrag()
 					m_End->GetCoords(&x2, &y2);
 					x2 *= m_dZoomFactor;
 					y2 *= m_dZoomFactor;
-					double d = sqrt(square(x2 - x1) + square(y2 - y1)), L = DefaultBondLength * m_dZoomFactor, t;
+					double d = sqrt(square(x2 - x1) + square(y2 - y1)), L = pDoc->GetBondLength () * m_dZoomFactor, t;
 					unsigned n = m_size - m_Chain->GetLength();
 					if (d < L * n)
 					{//FIXME: this algorithm is far from optimal!!!
@@ -421,8 +426,8 @@ void gcpCycleTool::OnDrag()
 				y1 = m_xn[3];
 				for (i = 4; i < m_size * 2; i+= 2)
 				{
-					m_xn[i] = x1 += (DefaultBondLength * m_dZoomFactor) * cos(m_dAngle - m_dDev * (i / 2 - 1));
-					m_xn[i + 1] = y1 -= (DefaultBondLength * m_dZoomFactor) * sin(m_dAngle - m_dDev * (i / 2 - 1));
+					m_xn[i] = x1 += (pDoc->GetBondLength () * m_dZoomFactor) * cos (m_dAngle - m_dDev * (i / 2 - 1));
+					m_xn[i + 1] = y1 -= (pDoc->GetBondLength () * m_dZoomFactor) * sin (m_dAngle - m_dDev * (i / 2 - 1));
 					points->coords[i] = x1;
 					points->coords[i + 1] = y1;
 				}
@@ -499,7 +504,7 @@ void gcpCycleTool::OnDrag()
 			m_dAngle = dAngle * M_PI / 180;
 			x1 = m_x0;
 			y1 = m_y0;
-			double d = (m_nState & GDK_SHIFT_MASK)? sqrt(square(m_x) + square(m_y)): DefaultBondLength * m_dZoomFactor;
+			double d = (m_nState & GDK_SHIFT_MASK)? sqrt (square (m_x) + square (m_y)): pDoc->GetBondLength () * m_dZoomFactor;
 			for (i = 2; i < m_size * 2; i+= 2)
 			{
 				m_xn[i] = x1 += d * cos(m_dAngle - m_dDev * (i / 2 - 1));
@@ -519,7 +524,7 @@ void gcpCycleTool::OnDrag()
 								gnome_canvas_polygon_get_type(),
 								"points", points,
 								"outline_color",  (m_bAllowed)? AddColor: DeleteColor,
-								"width_units", m_pData->BondWidth,
+								"width_units", pTheme->GetBondWidth (),
 								NULL);
 }
 
@@ -672,8 +677,9 @@ void gcpCycleTool::OnChangeState()
 			points->coords[3] = m_xn[3] = y1 = y2 * m_dZoomFactor;
 			for (int i = 4; i < m_size * 2; i+= 2)
 			{
-				m_xn[i] = x1 += (DefaultBondLength * m_dZoomFactor) * cos(m_dAngle - m_dDev * (i / 2 - 1));
-				m_xn[i + 1] = y1 -= (DefaultBondLength * m_dZoomFactor) * sin(m_dAngle - m_dDev * (i / 2 - 1));
+				gcpDocument *pDoc = m_pView->GetDoc ();
+				m_xn[i] = x1 += (pDoc->GetBondLength () * m_dZoomFactor) * cos(m_dAngle - m_dDev * (i / 2 - 1));
+				m_xn[i + 1] = y1 -= (pDoc->GetBondLength () * m_dZoomFactor) * sin(m_dAngle - m_dDev * (i / 2 - 1));
 				points->coords[i] = x1;
 				points->coords[i + 1] = y1;
 			}
@@ -738,9 +744,9 @@ bool gcpCycleTool::CheckIfAllowed()
 	return true;
 }
 
-static void on_length_changed (GtkSpinButton *btn)
+static void on_length_changed (GtkSpinButton *btn, gcpCycleTool *tool)
 {
-	DefaultBondLength = gtk_spin_button_get_value (btn);
+	tool->SetLength (gtk_spin_button_get_value (btn));
 }
 
 static void on_merge_toggled (GtkToggleButton *btn)
@@ -748,11 +754,16 @@ static void on_merge_toggled (GtkToggleButton *btn)
 	MergeAtoms = gtk_toggle_button_get_active (btn);
 }
 
+void gcpCycleTool::SetLength (double length)
+{
+	m_pApp->GetActiveDocument ()->SetBondLength (length);
+}
+
 GtkWidget *gcpCycleTool::GetPropertyPage ()
 {
 	GladeXML *xml = glade_xml_new (DATADIR"/gchempaint-unstable/ui/cycle.glade", "cycle", GETTEXT_PACKAGE);
 	m_LengthBtn = GTK_SPIN_BUTTON (glade_xml_get_widget (xml, "bond-length"));
-	g_signal_connect (m_LengthBtn, "value-changed", G_CALLBACK (on_length_changed), NULL);
+	g_signal_connect (m_LengthBtn, "value-changed", G_CALLBACK (on_length_changed), this);
 	m_MergeBtn = GTK_TOGGLE_BUTTON (glade_xml_get_widget (xml, "merge"));
 	g_signal_connect (m_MergeBtn, "toggled", G_CALLBACK (on_merge_toggled), NULL);
 	return glade_xml_get_widget (xml, "cycle");
@@ -762,7 +773,8 @@ extern bool new_ui;
 void gcpCycleTool::Activate ()
 {
 	if (new_ui) {
-		gtk_spin_button_set_value (m_LengthBtn, DefaultBondLength);
+		gcpDocument *pDoc = m_pApp->GetActiveDocument ();
+		gtk_spin_button_set_value (m_LengthBtn, pDoc->GetBondLength ());
 		gtk_toggle_button_set_active (m_MergeBtn, MergeAtoms);
 	}
 }
