@@ -2,7 +2,7 @@
  * GChemPaint selection plugin
  * group.cc
  *
- * Copyright (C) 2004-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004-2006 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -23,9 +23,10 @@
 #include "group.h"
 #include "groupdlg.h"
 #include <lib/document.h>
+#include <lib/theme.h>
 #include <lib/view.h>
 #include <lib/widgetdata.h>
-#include <cmath>
+#include <math.h>
 
 
 TypeId GroupType = NoType;
@@ -37,7 +38,7 @@ static void on_group_properties (gcpGroup* group)
 
 gcpGroup::gcpGroup (): Object(GroupType)
 {
-	SetId (const_cast<char*> ("gr1"));
+	SetId ("gr1");
 	m_Align = false;
 	m_Spaced = false;
 }
@@ -106,12 +107,14 @@ void gcpGroup::Align ()
 	map<string, Object*>::iterator i;
 	Object* obj = GetFirstChild (i);
 	ArtDRect rect;
-	gcpView * View = ((gcpDocument*) GetDocument ())->GetView ();
+	gcpDocument *pDoc = dynamic_cast <gcpDocument*> (GetDocument ());
+	gcpView *View = pDoc->GetView ();
+	gcpTheme *pTheme = pDoc->GetTheme ();
+	gcpWidgetData *Data = reinterpret_cast <gcpWidgetData*> (g_object_get_data (G_OBJECT (View->GetWidget ()), "data"));
 	double x = 0., t = 0.;
 	if ((m_AlignType == GCP_ALIGN_TOP) 
 		|| (m_AlignType == GCP_ALIGN_LEFT))
 		t = DBL_MAX;
-	gcpWidgetData *Data = (gcpWidgetData*) g_object_get_data (G_OBJECT (View->GetWidget ()), "data");
 	while (obj) {
 		if (m_AlignType == GCP_ALIGN_NORMAL) {
 			x = obj->GetYAlign ();
@@ -120,30 +123,30 @@ void gcpGroup::Align ()
 			Data->GetObjectBounds (obj, &rect);
 			switch (m_AlignType) {
 			case GCP_ALIGN_TOP:
-				x = rect.y0 / Data->ZoomFactor;
+				x = rect.y0 / pTheme->GetZoomFactor ();
 				if (t > x)
 					t = x;
 				break;
 			case GCP_ALIGN_MID_HEIGHT:
-				x = (rect.y0 + rect.y1) / 2. / Data->ZoomFactor;
+				x = (rect.y0 + rect.y1) / 2. / pTheme->GetZoomFactor ();
 				t += x;
 				break;
 			case GCP_ALIGN_BOTTOM:
-				x = rect.y1 / Data->ZoomFactor;
+				x = rect.y1 / pTheme->GetZoomFactor ();
 				if (t < x)
 					t = x;
 				break;
 			case GCP_ALIGN_LEFT:
-				x = rect.x0 / Data->ZoomFactor;
+				x = rect.x0 / pTheme->GetZoomFactor ();
 				if (t > x)
 					t = x;
 				break;
 			case GCP_ALIGN_CENTER:
-				x = (rect.x0 + rect.x1) / 2. / Data->ZoomFactor;
+				x = (rect.x0 + rect.x1) / 2. / pTheme->GetZoomFactor ();
 				t += x;
 				break;
 			case GCP_ALIGN_RIGHT:
-				x = rect.x1 / Data->ZoomFactor;
+				x = rect.x1 / pTheme->GetZoomFactor ();
 				if (t < x)
 					t = x;
 				break;
@@ -183,7 +186,9 @@ void gcpGroup::Space ()
 	Object* obj = GetFirstChild (i);
 	ArtDRect rect;
 	double x;
-	gcpView * View = ((gcpDocument*) GetDocument ())->GetView ();
+	gcpDocument *pDoc = dynamic_cast <gcpDocument*> (GetDocument ());
+	gcpView *View = pDoc->GetView ();
+	gcpTheme *pTheme = pDoc->GetTheme ();
 	gcpWidgetData *Data = (gcpWidgetData*) g_object_get_data (G_OBJECT (View->GetWidget ()), "data");
 	while (obj) {
 		Data->GetObjectBounds (obj, &rect);
@@ -198,17 +203,17 @@ void gcpGroup::Space ()
 	im = Children.begin();
 	rect = rects[(*im).second];
 	x = (m_AlignType <= GCP_ALIGN_BOTTOM)? rect.x1: rect.y1;
-	x /= Data->ZoomFactor;
+	x /= pTheme->GetZoomFactor ();
 	for (im++; im != endm; im++) {
 		x += m_Padding;
 		obj = (*im).second;
 		rect = rects[obj];
 		if (m_AlignType <= GCP_ALIGN_BOTTOM) {
-			obj->Move (x - rect.x0 / Data->ZoomFactor, 0);
-			x += (rect.x1 - rect.x0) / Data->ZoomFactor;
+			obj->Move (x - rect.x0 / pTheme->GetZoomFactor (), 0);
+			x += (rect.x1 - rect.x0) / pTheme->GetZoomFactor ();
 		} else {
-			obj->Move (0, x - rect.y0 / Data->ZoomFactor);
-			x += (rect.y1 - rect.y0) /Data->ZoomFactor;
+			obj->Move (0, x - rect.y0 / pTheme->GetZoomFactor ());
+			x += (rect.y1 - rect.y0) /pTheme->GetZoomFactor ();
 		}
 		View->Update (obj);
 	}
@@ -271,25 +276,25 @@ xmlNodePtr gcpGroup::Save (xmlDocPtr xml)
 		char *align_type = NULL;
 		switch (m_AlignType) {
 		case GCP_ALIGN_NORMAL:
-			align_type = const_cast<char*> ("normal");
+			align_type = "normal";
 			break;
 		case GCP_ALIGN_TOP:
-			align_type = const_cast<char*> ("top");
+			align_type = "top";
 			break;
 		case GCP_ALIGN_MID_HEIGHT:
-			align_type = const_cast<char*> ("mid-height");
+			align_type = "mid-height";
 			break;
 		case GCP_ALIGN_BOTTOM:
-			align_type = const_cast<char*> ("bottom");
+			align_type = "bottom";
 			break;
 		case GCP_ALIGN_LEFT:
-			align_type = const_cast<char*> ("left");
+			align_type = "left";
 			break;
 		case GCP_ALIGN_CENTER:
-			align_type = const_cast<char*> ("center");
+			align_type = "center";
 			break;
 		case GCP_ALIGN_RIGHT:
-			align_type = const_cast<char*> ("right");
+			align_type = "right";
 			break;
 		}
 		xmlNewProp (node, (const xmlChar*) "align", (const xmlChar*) align_type);
@@ -332,14 +337,3 @@ double gcpGroup::GetYAlign ()
 	pData->GetObjectBounds (this, &rect);
 	return (rect.y1 - rect.y0) / 2.;
 }
-
-void gcpGroup::Add (GtkWidget* w)
-{
-	map<string, Object*>::iterator i;
-	Object* p = GetFirstChild (i);
-	while (p)
-	{
-		p->Add (w);
-		p = GetNextChild (i);
-	}
-}
