@@ -42,6 +42,7 @@ public:
 	virtual void Activate();
 	virtual bool Deactivate();
 	virtual bool OnRightButtonClicked (GtkUIManager *UIManager);
+	virtual GtkWidget *GetPropertyPage ();
 
 	void AddSelection(gcpWidgetData* data);
 	void OnFlip (bool horizontal);
@@ -59,6 +60,8 @@ private:
 	gcpOperation* m_pOp;
 	TypeId m_Type;
 	list<int> m_uiIds;
+	GtkUIManager *m_UIManager;
+	GtkWidget *m_MergeBtn;
 };
 
 #endif // GCHEMPAINT_SELECTION_TOOL_H
