@@ -27,6 +27,7 @@
 #include "lib/fragment.h"
 #include "lib/document.h"
 #include "lib/application.h"
+#include "lib/theme.h"
 #include <gdk/gdkkeysyms.h>
 
 extern GtkTextTagTable *TextTagTable;
@@ -59,7 +60,8 @@ bool gcpFragmentTool::OnClicked()
 	if (!m_pObject)
 	{
 		gcpDocument* pDoc = m_pView->GetDoc();
-		gcpFragment *fragment = new gcpFragment(m_x0 / m_pData->ZoomFactor, m_y0 / m_pData->ZoomFactor);
+		gcpTheme *pTheme = pDoc->GetTheme ();
+		gcpFragment *fragment = new gcpFragment(m_x0 / pTheme->GetZoomFactor (), m_y0 / pTheme->GetZoomFactor ());
 		pDoc->AddFragment(fragment);
 		pDoc->AbortOperation();
 		pDoc->EmptyTranslationTable();
