@@ -28,6 +28,7 @@
 #include "lib/document.h"
 #include "lib/application.h"
 #include "lib/settings.h"
+#include "lib/theme.h"
 #include <gdk/gdkkeysyms.h>
 #include <unistd.h>
 
@@ -61,12 +62,6 @@ gcpTextTool::gcpTextTool (gcpApplication* App, string Id): gcpTool (App, Id)
 		m_pApp->SetCallback ("font", (GCallback) on_font);
 	}
 	m_FontDesc = NULL;
-	m_Style = DefaultFontStyle;
-	m_Weight = DefaultFontWeight;
-	m_Stretch = DefaultFontStretch;
-	m_Variant = DefaultFontVariant;
-	m_Size = DefaultFontSize;
-	m_Scalable = false; // force sizes list creation
 }
 
 gcpTextTool::~gcpTextTool ()
@@ -89,7 +84,8 @@ bool gcpTextTool::OnClicked()
 	}
 	if (!m_pObject)
 	{
-		gcpText *text = new gcpText(m_x0 / m_pData->ZoomFactor, m_y0 / m_pData->ZoomFactor);
+		gcpTheme *pTheme = m_pView->GetDoc ()->GetTheme ();
+		gcpText *text = new gcpText(m_x0 / pTheme->GetZoomFactor (), m_y0 / pTheme->GetZoomFactor ());
 		m_pView->GetDoc()->AddObject(text);
 		m_pView->GetDoc()->AbortOperation();
 		m_pObject = text;
@@ -234,6 +230,12 @@ void gcpTextTool::Activate()
 	m_pApp->ActivateToolItem ("subscript", true);
 	m_pApp->ActivateToolItem ("superscript", true);
 	m_pApp->ActivateToolItem ("Font", true);
+	gcpTheme *pTheme = m_pApp->GetActiveDocument ()->GetTheme ();
+	m_Style = pTheme->GetFontStyle ();
+	m_Weight = pTheme->GetFontWeight ();
+	m_Stretch = pTheme->GetFontStretch ();
+	m_Variant = pTheme->GetFontVariant ();
+	m_Size = pTheme->GetFontSize ();
 }
 
 bool gcpTextTool::Deactivate()
@@ -692,6 +694,13 @@ static void on_size_focus_out (GtkEntry *entry, GdkEventFocus *event, gcpTextToo
 {
 }
 
+/* These are what we use as the standard font sizes, for the size list.
+ */
+static const guint16 font_sizes[] = {
+  8, 9, 10, 11, 12, 13, 14, 16, 18, 20, 22, 24, 26, 28,
+  32, 36, 40, 48, 56, 64, 72
+};
+
 GtkWidget *gcpTextTool::GetPropertyPage ()
 {
 	GladeXML *xml = glade_xml_new (DATADIR"/gchempaint-unstable/ui/fontsel.glade", "fontsel", GETTEXT_PACKAGE);
@@ -699,6 +708,7 @@ GtkWidget *gcpTextTool::GetPropertyPage ()
 	gtk_combo_box_set_active (GTK_COMBO_BOX (w), 0); // FIXME: set the current underline style
 	PangoFontFamily **families;
 	int i, nb;
+	gcpTheme *pTheme = m_pApp->GetActiveDocument ()->GetTheme ();
 	// Initialize faces list
 	m_FaceList = gtk_list_store_new (1, G_TYPE_STRING);
 	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (m_FaceList), 0, GTK_SORT_ASCENDING);
@@ -717,6 +727,13 @@ GtkWidget *gcpTextTool::GetPropertyPage ()
 	renderer = gtk_cell_renderer_text_new ();
 	column = gtk_tree_view_column_new_with_attributes (NULL, renderer, "text", 0, NULL);
 	gtk_tree_view_append_column (tree, column);
+	GtkTreeIter iter, selected;
+	for (i = 0; i < (int) G_N_ELEMENTS (font_sizes); i++) {
+		gtk_list_store_append (m_SizeList, &iter);
+		gtk_list_store_set (m_SizeList, &iter,
+				  0, font_sizes[i],
+				  -1);
+	}
 	selection = gtk_tree_view_get_selection (tree);
 	m_SizeSel = selection;
 	m_SizeSignal = g_signal_connect (m_SizeSel, "changed", G_CALLBACK (on_select_size), this);
@@ -733,20 +750,33 @@ GtkWidget *gcpTextTool::GetPropertyPage ()
 	renderer = gtk_cell_renderer_text_new ();
 	column = gtk_tree_view_column_new_with_attributes (NULL, renderer, "text", 0, NULL);
 	gtk_tree_view_append_column (tree, column);
-	GtkTreeIter iter, selected;
 	GtkTreePath *path = NULL;
 	string name;
-	gtk_tree_model_get_iter_first   (GTK_TREE_MODEL (m_FamilyList), &selected);
+	bool default_found = false;
 	for (i = 0; i < nb; i++) {
+		PangoFontFace **faces;
+		int *sizes, n;
+		pango_font_family_list_faces (families[i], &faces, &n);
+		if (n <= 0)
+			continue;
+		pango_font_face_list_sizes (faces[0], &sizes, &n);
+		if (n > 0) // Do not use bitmap fonts
+			continue;
 		name = pango_font_family_get_name (families[i]);
 		m_Families[name] = (PangoFontFamily*) g_object_ref (families[i]);
 		gtk_list_store_append (m_FamilyList, &iter);
 		gtk_list_store_set (m_FamilyList, &iter,
 				  0, name.c_str (),
 				  -1);
-		if (name == DefaultTextFontFamily)
+		if (name == pTheme->GetTextFontFamily ()) {
 			selected = iter;
-//			path = gtk_tree_model_get_path (GTK_TREE_MODEL (m_FamilyList), &iter);
+			default_found = true;
+		}
+	}
+	if (!default_found) {
+		if	(!gtk_tree_model_get_iter_first (GTK_TREE_MODEL (m_FamilyList), &iter))
+			return NULL;
+		selected = iter;
 	}
 	path = gtk_tree_model_get_path (GTK_TREE_MODEL (m_FamilyList), &selected);
 	selection = gtk_tree_view_get_selection (tree);
@@ -757,8 +787,6 @@ GtkWidget *gcpTextTool::GetPropertyPage ()
 		gtk_tree_selection_select_path (selection, path);
 		gtk_tree_view_scroll_to_cell (tree, path, column, FALSE, 0., 0.);
 		gtk_tree_path_free (path);
-	} else {
-		//TODO: choose a font when default is not available
 	}
 
 	return glade_xml_get_widget (xml, "fontsel");
@@ -825,20 +853,11 @@ void gcpTextTool::OnSelectFamily (GtkTreeSelection *selection)
 	}
 }
 
-/* These are what we use as the standard font sizes, for the size list.
- */
-static const guint16 font_sizes[] = {
-  8, 9, 10, 11, 12, 13, 14, 16, 18, 20, 22, 24, 26, 28,
-  32, 36, 40, 48, 56, 64, 72
-};
-
 void gcpTextTool::OnSelectFace (GtkTreeSelection *selection)
 {
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 	char const *name;
-	g_signal_handler_block (m_SizeSel, m_SizeSignal);
-	m_Dirty = true;
 	if (!gtk_tree_selection_get_selected (selection, &model, &iter))
 		return;
 	gtk_tree_model_get (model, &iter, 0, &name, -1);
@@ -849,27 +868,7 @@ void gcpTextTool::OnSelectFace (GtkTreeSelection *selection)
 	m_Variant = pango_font_description_get_variant (desc);
 	m_Stretch = pango_font_description_get_stretch (desc);
 	pango_font_description_free (desc);
-	// Populate sizes
-	int i, nb, *sizes;
-	pango_font_face_list_sizes (face, &sizes, &nb);
-	if (nb == 0) {
-		if (!m_Scalable) {
-			m_Scalable = true;
-			gtk_list_store_clear (m_SizeList);
-			for (i = 0; i < (int) G_N_ELEMENTS (font_sizes); i++) {
-				gtk_list_store_append (m_SizeList, &iter);
-				gtk_list_store_set (m_SizeList, &iter,
-						  0, font_sizes[i],
-						  -1);
-			}
-		}
-	} else {
-		m_Scalable = false;
-		gtk_list_store_clear (m_SizeList);
-	}
-	g_signal_handler_unblock (m_SizeSel, m_SizeSignal);
-	if (m_Dirty)
-		BuildAttributeList ();
+	BuildAttributeList ();
 }
 
 void gcpTextTool::OnSelectSize (int size)
