@@ -226,12 +226,12 @@ static void on_help (GtkWidget *widget, gcpStandaloneApp* App)
 
 static void on_web (GtkWidget *widget, gcpStandaloneApp* App)
 {
-	App->OnWeb ();
+	App->OnWeb ("gchempaint.nongnu.org");
 }
 
 static void on_mail (GtkWidget *widget, gcpStandaloneApp* App)
 {
-	App->OnMail ();
+	App->OnMail ("mailto:gchempaint-main@nongnu.org");
 }
 
 static void on_bug (GtkWidget *widget, gcpStandaloneApp* App)
@@ -426,8 +426,6 @@ static const char *ui_description =
 "    <toolitem action='Open'/>"
 "    <toolitem action='Save'/>"
 "    <toolitem action='Print'/>"
-"    <toolitem action='Close'/>"
-"    <toolitem action='Quit'/>"
 "  </toolbar>"
 "</ui>";
 
@@ -613,7 +611,7 @@ void gcpStandaloneApp::OnFileNew()
 	if (new_ui) {
 		gcpWindow *win = new gcpStandaloneWindow (this);
 		win->SetTitle (tmp);
-		m_Docs.push_back (win->GetDocument ());
+//		m_Docs.push_back (win->GetDocument ());
 	} else {
 		GtkWidget* w = CreateView ();
 		if (w) {
@@ -654,7 +652,7 @@ void gcpStandaloneApp::OnFileOpen()
 	FileChooser (this, false, GetSupportedMimeTypes ());
 }
 
-void  gcpStandaloneApp::AddDocument(gcpDocument* pDoc)
+/*void  gcpStandaloneApp::AddDocument(gcpDocument* pDoc)
 {
 	if (!pDoc) return;
 	if (pDoc)
@@ -708,14 +706,14 @@ void gcpStandaloneApp::RemoveDocument(gcpDocument* pDoc)
 	if (new_ui && m_Docs.size () == 0) {
 		gtk_main_quit ();
 	}
-}
+}*/
 
 GtkWidget* gcpStandaloneApp::CreateView()
 {
 	gcpDocument* pDoc = new gcpDocument(this, true);
 	if (pDoc)
 	{
-		m_Docs.push_back(pDoc);
+//		m_Docs.push_back(pDoc);
 		m_pActiveDoc = pDoc;
 		return pDoc->GetWidget();
 	}
@@ -1010,6 +1008,8 @@ void gcpStandaloneApp::SetActive (gcpDocument* pDoc, GtkWidget* w)
 
 void gcpStandaloneApp::OnFileClose ()
 {
+	if (!m_pActiveDoc)
+		return;
 	gcpView* pView = m_pActiveDoc->GetView ();
 	pView->PrepareUnselect (); 
 	m_pActiveDoc = NULL;
@@ -1021,6 +1021,8 @@ void gcpStandaloneApp::OnFileClose ()
 bool gcpStandaloneApp::OnKeyPressed(GtkWidget* widget, GdkEventKey* ev)
 {
 	gint i = gtk_notebook_get_current_page(m_Book);
+	if (i < 0)
+		return true;
 	GtkWidget* w = gtk_bin_get_child(GTK_BIN(gtk_bin_get_child(GTK_BIN(gtk_notebook_get_nth_page(m_Book, i)))));
 	gcpView* v = (gcpView*) g_object_get_data(G_OBJECT(w), "view");
 	if (v && v->GetDoc()->GetEditable()) return v->OnKeyPress(w, ev);
@@ -1030,6 +1032,8 @@ bool gcpStandaloneApp::OnKeyPressed(GtkWidget* widget, GdkEventKey* ev)
 bool gcpStandaloneApp::OnKeyReleased(GtkWidget* widget, GdkEventKey* ev)
 {
 	gint i = gtk_notebook_get_current_page(m_Book);
+	if (i < 0)
+		return false;
 	GtkWidget* w = gtk_bin_get_child(GTK_BIN(gtk_bin_get_child(GTK_BIN(gtk_notebook_get_nth_page(m_Book, i)))));
 	gcpView* v = (gcpView*) g_object_get_data(G_OBJECT(w), "view");
 	if (v && v->GetDoc()->GetEditable()) return v->OnKeyRelease(w, ev);
@@ -1100,12 +1104,13 @@ void gcpStandaloneApp::OnCloseAll ()
 
 void gcpStandaloneApp::OnSaveAll ()
 {
-	gcpDocument* pDoc = m_pActiveDoc;
-	list<gcpDocument*>::iterator i, end = m_Docs.end ();
+	gcpDocument* pDoc = m_pActiveDoc, *pCurDoc;
+	set<Document*>::iterator i, end = m_Docs.end ();
 	for (i = m_Docs.begin (); i != end; i++) {
-		if (!(*i)->IsDirty ())
+		pCurDoc = dynamic_cast <gcpDocument *> (*i);
+		if (!pCurDoc->IsDirty ())
 			continue;
-		on_select (NULL, (*i)->GetWidget ());
+		on_select (NULL, pCurDoc->GetWidget ());
 		OnSave ();
 	}
 	if (pDoc)
@@ -1125,3 +1130,9 @@ void gcpStandaloneApp::Show ()
 {
 	gtk_widget_show_all (GTK_WIDGET (m_Window));
 }
+
+void gcpStandaloneApp::NoMoreDocsEvent ()
+{
+	if (new_ui)
+		gtk_main_quit ();
+}
